import sys
import typing
import bpy_types
import rna_prop_ui
import bl_ui.properties_animviz

GenericType = typing.TypeVar("GenericType")


class COLLECTION_MT_context_menu(bpy_types.Menu, bpy_types._GenericUI):
    bl_label = None
    ''' '''

    bl_rna = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, _context):
        ''' 

        '''
        pass

    def draw_collapsible(self, context, layout):
        ''' 

        '''
        pass

    def draw_preset(self, _context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_menu(self, searchpaths, operator, props_default, prop_filepath,
                  filter_ext, filter_path, display_name, add_operator):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_motion_paths(bl_ui.properties_animviz.MotionPathButtonsPanel,
                             bpy_types.Panel, bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def draw_settings(self, _context, avs, mpath, bones):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_motion_paths_display(
        bl_ui.properties_animviz.MotionPathButtonsPanel_display,
        bpy_types.Panel, bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_parent_id = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def draw_settings(self, _context, avs, mpath, bones):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class ObjectButtonsPanel:
    bl_context = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_space_type = None
    ''' '''


class OBJECT_PT_collections(ObjectButtonsPanel, bpy_types.Panel,
                            bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_context_object(ObjectButtonsPanel, bpy_types.Panel,
                               bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_custom_props(ObjectButtonsPanel, rna_prop_ui.PropertyPanel,
                             bpy_types.Panel, bpy_types._GenericUI):
    COMPAT_ENGINES = None
    ''' '''

    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_order = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_delta_transform(ObjectButtonsPanel, bpy_types.Panel,
                                bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_parent_id = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_display(ObjectButtonsPanel, bpy_types.Panel,
                        bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_order = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_instancing(ObjectButtonsPanel, bpy_types.Panel,
                           bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_instancing_size(ObjectButtonsPanel, bpy_types.Panel,
                                bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_parent_id = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def draw_header(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_lineart(ObjectButtonsPanel, bpy_types.Panel,
                        bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_order = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_relations(ObjectButtonsPanel, bpy_types.Panel,
                          bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_transform(ObjectButtonsPanel, bpy_types.Panel,
                          bpy_types._GenericUI):
    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


class OBJECT_PT_visibility(ObjectButtonsPanel, bpy_types.Panel,
                           bpy_types._GenericUI):
    COMPAT_ENGINES = None
    ''' '''

    bl_context = None
    ''' '''

    bl_label = None
    ''' '''

    bl_options = None
    ''' '''

    bl_region_type = None
    ''' '''

    bl_rna = None
    ''' '''

    bl_space_type = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def id_properties_clear(self):
        ''' 

        '''
        pass

    def id_properties_ensure(self):
        ''' 

        '''
        pass

    def id_properties_ui(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def poll(self, context):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass



import sys
import typing

GenericType = typing.TypeVar("GenericType")


class Device:
    ''' Device objects represent an audio output backend like OpenAL or SDL, but might also represent a file output or RAM buffer output.
    '''

    channels = None
    ''' The channel count of the device.'''

    distance_model = None
    ''' The distance model of the device.'''

    doppler_factor = None
    ''' The doppler factor of the device. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.'''

    format = None
    ''' The native sample format of the device.'''

    listener_location = None
    ''' The listeners's location in 3D space, a 3D tuple of floats.'''

    listener_orientation = None
    ''' The listener's orientation in 3D space as quaternion, a 4 float tuple.'''

    listener_velocity = None
    ''' The listener's velocity in 3D space, a 3D tuple of floats.'''

    rate = None
    ''' The sampling rate of the device in Hz.'''

    speed_of_sound = None
    ''' The speed of sound of the device. The speed of sound in air is typically 343.3 m/s.'''

    volume = None
    ''' The overall volume of the device.'''

    @classmethod
    def lock(cls):
        ''' Locks the device so that it's guaranteed, that no samples are read from the streams until :meth:`unlock` is called. This is useful if you want to do start/stop/pause/resume some sounds at the same time.

        '''
        pass

    @classmethod
    def play(cls, sound: 'Sound', keep: bool = False) -> 'Handle':
        ''' Plays a sound.

        :param sound: The sound to play.
        :type sound: 'Sound'
        :param keep: `Handle.keep`.
        :type keep: bool
        :rtype: 'Handle'
        :return: The playback handle with which playback can be controlled with.
        '''
        pass

    @classmethod
    def stopAll(cls):
        ''' Stops all playing and paused sounds.

        '''
        pass

    @classmethod
    def unlock(cls):
        ''' Unlocks the device after a lock call, see :meth:`lock` for details.

        '''
        pass


class DynamicMusic:
    ''' The DynamicMusic object allows to play music depending on a current scene, scene changes are managed by the class, with the possibility of custom transitions. The default transition is a crossfade effect, and the default scene is silent and has id 0
    '''

    fadeTime = None
    ''' The length in seconds of the crossfade transition'''

    position = None
    ''' The playback position of the scene in seconds.'''

    scene = None
    ''' The current scene'''

    status = None
    ''' Whether the scene is playing, paused or stopped (=invalid).'''

    volume = None
    ''' The volume of the scene.'''

    @classmethod
    def addScene(cls, scene: 'Sound') -> int:
        ''' Adds a new scene.

        :param scene: The scene sound.
        :type scene: 'Sound'
        :rtype: int
        :return: The new scene id.
        '''
        pass

    @classmethod
    def addTransition(cls, ini: int, end: int, transition: 'Sound') -> bool:
        ''' Adds a new scene.

        :param ini: the initial scene foor the transition.
        :type ini: int
        :param end: The final scene for the transition.
        :type end: int
        :param transition: The transition sound.
        :type transition: 'Sound'
        :rtype: bool
        :return: false if the ini or end scenes don't exist, true othrwise.
        '''
        pass

    @classmethod
    def pause(cls) -> bool:
        ''' Pauses playback of the scene.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def resume(cls) -> bool:
        ''' Resumes playback of the scene.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def stop(cls) -> bool:
        ''' Stops playback of the scene.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass


class Handle:
    ''' Handle objects are playback handles that can be used to control playback of a sound. If a sound is played back multiple times then there are as many handles.
    '''

    attenuation = None
    ''' This factor is used for distance based attenuation of the source.'''

    cone_angle_inner = None
    ''' The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr:`location` of the source and with infinite height, heading in the direction of the source's :attr:`orientation`. In the inner cone the volume is normal. Outside the outer cone the volume will be :attr:`cone_volume_outer` and in the area between the volume will be interpolated linearly.'''

    cone_angle_outer = None
    ''' The opening angle of the outer cone of the source.'''

    cone_volume_outer = None
    ''' The volume outside the outer cone of the source.'''

    distance_maximum = None
    ''' The maximum distance of the source. If the listener is further away the source volume will be 0.'''

    distance_reference = None
    ''' The reference distance of the source. At this distance the volume will be exactly :attr:`volume`.'''

    keep = None
    ''' Whether the sound should be kept paused in the device when its end is reached. This can be used to seek the sound to some position and start playback again.'''

    location = None
    ''' The source's location in 3D space, a 3D tuple of floats.'''

    loop_count = None
    ''' The (remaining) loop count of the sound. A negative value indicates infinity.'''

    orientation = None
    ''' The source's orientation in 3D space as quaternion, a 4 float tuple.'''

    pitch = None
    ''' The pitch of the sound.'''

    position = None
    ''' The playback position of the sound in seconds.'''

    relative = None
    ''' Whether the source's location, velocity and orientation is relative or absolute to the listener.'''

    status = None
    ''' Whether the sound is playing, paused or stopped (=invalid).'''

    velocity = None
    ''' The source's velocity in 3D space, a 3D tuple of floats.'''

    volume = None
    ''' The volume of the sound.'''

    volume_maximum = None
    ''' The maximum volume of the source.'''

    volume_minimum = None
    ''' The minimum volume of the source.'''

    @classmethod
    def pause(cls) -> bool:
        ''' Pauses playback.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def resume(cls) -> bool:
        ''' Resumes playback.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def stop(cls) -> bool:
        ''' Stops playback.

        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass


class PlaybackManager:
    ''' A PlabackManager object allows to easily control groups os sounds organized in categories.
    '''

    @classmethod
    def addCategory(cls, volume: float) -> int:
        ''' Adds a category with a custom volume.

        :param volume: The volume for ther new category.
        :type volume: float
        :rtype: int
        :return: The key of the new category.
        '''
        pass

    @classmethod
    def clean(cls):
        ''' Cleans all the invalid and finished sound from the playback manager.

        '''
        pass

    @classmethod
    def getVolume(cls, catKey: int) -> float:
        ''' Retrieves the volume of a category.

        :param catKey: the key of the category.
        :type catKey: int
        :rtype: float
        :return: The volume of the cateogry.
        '''
        pass

    @classmethod
    def pause(cls, catKey: int) -> bool:
        ''' Pauses playback of the category.

        :param catKey: the key of the category.
        :type catKey: int
        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def play(cls, sound: 'Sound', catKey: int) -> 'Handle':
        ''' Plays a sound through the playback manager and assigns it to a category.

        :param sound: The sound to play.
        :type sound: 'Sound'
        :param catKey: the key of the category in which the sound will be added, if it doesn't exist, a new one will be created.
        :type catKey: int
        :rtype: 'Handle'
        :return: The playback handle with which playback can be controlled with.
        '''
        pass

    @classmethod
    def resume(cls, catKey: int) -> bool:
        ''' Resumes playback of the catgory.

        :param catKey: the key of the category.
        :type catKey: int
        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def setVolume(cls, volume: float, catKey: int) -> int:
        ''' Changes the volume of a category.

        :param volume: the new volume value.
        :type volume: float
        :param catKey: the key of the category.
        :type catKey: int
        :rtype: int
        :return: Whether the action succeeded.
        '''
        pass

    @classmethod
    def stop(cls, catKey: int) -> bool:
        ''' Stops playback of the category.

        :param catKey: the key of the category.
        :type catKey: int
        :rtype: bool
        :return: Whether the action succeeded.
        '''
        pass


class Sequence:
    ''' This sound represents sequenced entries to play a sound sequence.
    '''

    channels = None
    ''' The channel count of the sequence.'''

    distance_model = None
    ''' The distance model of the sequence.'''

    doppler_factor = None
    ''' The doppler factor of the sequence. This factor is a scaling factor for the velocity vectors in doppler calculation. So a value bigger than 1 will exaggerate the effect as it raises the velocity.'''

    fps = None
    ''' The listeners's location in 3D space, a 3D tuple of floats.'''

    muted = None
    ''' Whether the whole sequence is muted.'''

    rate = None
    ''' The sampling rate of the sequence in Hz.'''

    speed_of_sound = None
    ''' The speed of sound of the sequence. The speed of sound in air is typically 343.3 m/s.'''

    @classmethod
    def add(cls) -> 'SequenceEntry':
        ''' Adds a new entry to the sequence.

        :param sound: The sound this entry should play.
        :type sound: 'Sound'
        :param begin: The start time.
        :type begin: float
        :param end: The end time or a negative value if determined by the sound.
        :type end: float
        :param skip: How much seconds should be skipped at the beginning.
        :type skip: float
        :rtype: 'SequenceEntry'
        :return: The entry added.
        '''
        pass

    @classmethod
    def remove(cls):
        ''' Removes an entry from the sequence.

        :param entry: The entry to remove.
        :type entry: 'SequenceEntry'
        '''
        pass

    @classmethod
    def setAnimationData(cls):
        ''' Writes animation data to a sequence.

        :param type: The type of animation data.
        :type type: int
        :param frame: The frame this data is for.
        :type frame: int
        :param data: The data to write.
        :type data: typing.List[float]
        :param animated: Whether the attribute is animated.
        :type animated: bool
        '''
        pass


class SequenceEntry:
    ''' SequenceEntry objects represent an entry of a sequenced sound.
    '''

    attenuation = None
    ''' This factor is used for distance based attenuation of the source.'''

    cone_angle_inner = None
    ''' The opening angle of the inner cone of the source. If the cone values of a source are set there are two (audible) cones with the apex at the :attr:`location` of the source and with infinite height, heading in the direction of the source's :attr:`orientation`. In the inner cone the volume is normal. Outside the outer cone the volume will be :attr:`cone_volume_outer` and in the area between the volume will be interpolated linearly.'''

    cone_angle_outer = None
    ''' The opening angle of the outer cone of the source.'''

    cone_volume_outer = None
    ''' The volume outside the outer cone of the source.'''

    distance_maximum = None
    ''' The maximum distance of the source. If the listener is further away the source volume will be 0.'''

    distance_reference = None
    ''' The reference distance of the source. At this distance the volume will be exactly :attr:`volume`.'''

    muted = None
    ''' Whether the entry is muted.'''

    relative = None
    ''' Whether the source's location, velocity and orientation is relative or absolute to the listener.'''

    sound = None
    ''' The sound the entry is representing and will be played in the sequence.'''

    volume_maximum = None
    ''' The maximum volume of the source.'''

    volume_minimum = None
    ''' The minimum volume of the source.'''

    @classmethod
    def move(cls):
        ''' Moves the entry.

        :param begin: The new start time.
        :type begin: float
        :param end: The new end time or a negative value if unknown.
        :type end: float
        :param skip: How many seconds to skip at the beginning.
        :type skip: float
        '''
        pass

    @classmethod
    def setAnimationData(cls):
        ''' Writes animation data to a sequenced entry.

        :param type: The type of animation data.
        :type type: int
        :param frame: The frame this data is for.
        :type frame: int
        :param data: The data to write.
        :type data: typing.List[float]
        :param animated: Whether the attribute is animated.
        :type animated: bool
        '''
        pass


class Sound:
    ''' Sound objects are immutable and represent a sound that can be played simultaneously multiple times. They are called factories because they create reader objects internally that are used for playback.
    '''

    length = None
    ''' The sample specification of the sound as a tuple with rate and channel count.'''

    specs = None
    ''' The sample specification of the sound as a tuple with rate and channel count.'''

    @classmethod
    def buffer(cls, data: typing.Any, rate: float) -> 'Sound':
        ''' Creates a sound from a data buffer.

        :param data: The data as two dimensional numpy array.
        :type data: typing.Any
        :param rate: The sample rate.
        :type rate: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def file(cls, filename: str) -> 'Sound':
        ''' Creates a sound object of a sound file.

        :param filename: Path of the file.
        :type filename: str
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def list(cls) -> 'Sound':
        ''' Creates an empty sound list that can contain several sounds.

        :param random: whether the playback will be random or not.
        :type random: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def sawtooth(cls, frequency: float, rate: int = 48000) -> 'Sound':
        ''' Creates a sawtooth sound which plays a sawtooth wave.

        :param frequency: The frequency of the sawtooth wave in Hz.
        :type frequency: float
        :param rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.
        :type rate: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def silence(cls, rate: int = 48000) -> 'Sound':
        ''' Creates a silence sound which plays simple silence.

        :param rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.
        :type rate: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def sine(cls, frequency: float, rate: int = 48000) -> 'Sound':
        ''' Creates a sine sound which plays a sine wave.

        :param frequency: The frequency of the sine wave in Hz.
        :type frequency: float
        :param rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.
        :type rate: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def square(cls, frequency: float, rate: int = 48000) -> 'Sound':
        ''' Creates a square sound which plays a square wave.

        :param frequency: The frequency of the square wave in Hz.
        :type frequency: float
        :param rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.
        :type rate: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def triangle(cls, frequency: float, rate: int = 48000) -> 'Sound':
        ''' Creates a triangle sound which plays a triangle wave.

        :param frequency: The frequency of the triangle wave in Hz.
        :type frequency: float
        :param rate: The sampling rate in Hz. It's recommended to set this value to the playback device's samling rate to avoid resamping.
        :type rate: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def ADSR(cls, attack: float, decay: float, sustain: float,
             release: float) -> 'Sound':
        ''' Attack-Decay-Sustain-Release envelopes the volume of a sound. Note: there is currently no way to trigger the release with this API.

        :param attack: The attack time in seconds.
        :type attack: float
        :param decay: The decay time in seconds.
        :type decay: float
        :param sustain: The sustain level.
        :type sustain: float
        :param release: The release level.
        :type release: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def accumulate(cls, additive: typing.Any = False) -> 'Sound':
        ''' Accumulates a sound by summing over positive input differences thus generating a monotonic sigal. If additivity is set to true negative input differences get added too, but positive ones with a factor of two. Note that with additivity the signal is not monotonic anymore.

        :param time: 
        :type time: bool
        :param additive:  Whether the accumulation should be additive or not.
        :type additive: typing.Any
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def addSound(cls, sound: 'Sound'):
        ''' Adds a new sound to a sound list.

        :param sound: The sound that will be added to the list.
        :type sound: 'Sound'
        '''
        pass

    @classmethod
    def cache(cls) -> 'Sound':
        ''' Caches a sound into RAM. This saves CPU usage needed for decoding and file access if the underlying sound reads from a file on the harddisk, but it consumes a lot of memory.

        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def data(cls) -> typing.Any:
        ''' Retrieves the data of the sound as numpy array.

        :rtype: typing.Any
        :return: A two dimensional numpy float array.
        '''
        pass

    @classmethod
    def delay(cls, time: float) -> 'Sound':
        ''' Delays by playing adding silence in front of the other sound's data.

        :param time: How many seconds of silence should be added before the sound.
        :type time: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def envelope(cls, attack: float, release: float, threshold: float,
                 arthreshold: float) -> 'Sound':
        ''' Delays by playing adding silence in front of the other sound's data.

        :param attack: The attack factor.
        :type attack: float
        :param release: The release factor.
        :type release: float
        :param threshold: The general threshold value.
        :type threshold: float
        :param arthreshold: The attack/release threshold value.
        :type arthreshold: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def fadein(cls, start: float, length: float) -> 'Sound':
        ''' Fades a sound in by raising the volume linearly in the given time interval.

        :param start: Time in seconds when the fading should start.
        :type start: float
        :param length: Time in seconds how long the fading should last.
        :type length: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def fadeout(cls, start: float, length: float) -> 'Sound':
        ''' Fades a sound in by lowering the volume linearly in the given time interval.

        :param start: Time in seconds when the fading should start.
        :type start: float
        :param length: Time in seconds how long the fading should last.
        :type length: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def filter(cls, b: typing.List[float], a=(1)) -> 'Sound':
        ''' Filters a sound with the supplied IIR filter coefficients. Without the second parameter you'll get a FIR filter. If the first value of the a sequence is 0, it will be set to 1 automatically. If the first value of the a sequence is neither 0 nor 1, all filter coefficients will be scaled by this value so that it is 1 in the end, you don't have to scale yourself.

        :param b: The nominator filter coefficients.
        :type b: typing.List[float]
        :param a: The denominator filter coefficients.
        :type a: typing.List[float]
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def highpass(cls, frequency: float, Q: float = 0.5) -> 'Sound':
        ''' Creates a second order highpass filter based on the transfer function :math:`H(s) = s^2 / (s^2 + s/Q + 1)`

        :param frequency: The cut off trequency of the highpass.
        :type frequency: float
        :param Q: Q factor of the lowpass.
        :type Q: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def join(cls, sound: 'Sound') -> 'Sound':
        ''' Plays two factories in sequence.

        :param sound: The sound to play second.
        :type sound: 'Sound'
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def limit(cls, start: float, end: float) -> 'Sound':
        ''' Limits a sound within a specific start and end time.

        :param start: Start time in seconds.
        :type start: float
        :param end: End time in seconds.
        :type end: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def loop(cls, count: typing.Any) -> 'Sound':
        ''' Loops a sound.

        :param count: How often the sound should be looped. Negative values mean endlessly.
        :type count: typing.Any
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def lowpass(cls, frequency: float, Q: float = 0.5) -> 'Sound':
        ''' Creates a second order lowpass filter based on the transfer function :math:`H(s) = 1 / (s^2 + s/Q + 1)`

        :param frequency: The cut off trequency of the lowpass.
        :type frequency: float
        :param Q: Q factor of the lowpass.
        :type Q: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def mix(cls, sound: 'Sound') -> 'Sound':
        ''' Mixes two factories.

        :param sound: The sound to mix over the other.
        :type sound: 'Sound'
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def modulate(cls, sound: 'Sound') -> 'Sound':
        ''' Modulates two factories.

        :param sound: The sound to modulate over the other.
        :type sound: 'Sound'
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def mutable(cls) -> 'Sound':
        ''' Creates a sound that will be restarted when sought backwards. If the original sound is a sound list, the playing sound can change.

        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def pingpong(cls) -> 'Sound':
        ''' Plays a sound forward and then backward. This is like joining a sound with its reverse.

        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def pitch(cls, factor: float) -> 'Sound':
        ''' Changes the pitch of a sound with a specific factor.

        :param factor: The factor to change the pitch with.
        :type factor: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def rechannel(cls, channels: int) -> 'Sound':
        ''' Rechannels the sound.

        :param channels: The new channel configuration.
        :type channels: int
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def resample(cls, rate: float, high_quality: bool) -> 'Sound':
        ''' Resamples the sound.

        :param rate: The new sample rate.
        :type rate: float
        :param high_quality: When true use a higher quality but slower resampler.
        :type high_quality: bool
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def reverse(cls) -> 'Sound':
        ''' Plays a sound reversed.

        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def sum(cls) -> 'Sound':
        ''' Sums the samples of a sound.

        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def threshold(cls, threshold=' 0') -> 'Sound':
        ''' Makes a threshold wave out of an audio wave by setting all samples with a amplitude >= threshold to 1, all <= -threshold to -1 and all between to 0.

        :param threshold: Threshold value over which an amplitude counts non-zero.
        :type threshold: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def volume(cls, volume: float) -> 'Sound':
        ''' Changes the volume of a sound.

        :param volume: The new volume..
        :type volume: float
        :rtype: 'Sound'
        :return: `Sound` object.
        '''
        pass

    @classmethod
    def write(cls, filename: str, rate: int, channels: int, format: int,
              container: int, codec: int, bitrate: int, buffersize: int):
        ''' Writes the sound to a file.

        :param filename: The path to write to.
        :type filename: str
        :param rate: The sample rate to write with.
        :type rate: int
        :param channels: The number of channels to write with.
        :type channels: int
        :param format: The sample format to write with.
        :type format: int
        :param container: The container format for the file.
        :type container: int
        :param codec: The codec to use in the file.
        :type codec: int
        :param bitrate: The bitrate to write with.
        :type bitrate: int
        :param buffersize: The size of the writing buffer.
        :type buffersize: int
        '''
        pass


class Source:
    ''' The source object represents the source position of a binaural sound.
    '''

    azimuth = None
    ''' The azimuth angle.'''

    distance = None
    ''' The distance value. 0 is min, 1 is max.'''

    elevation = None
    ''' The elevation angle.'''


class ThreadPool:
    ''' A ThreadPool is used to parallelize convolution efficiently.
    '''

    pass


class error:
    pass


AP_LOCATION = None
''' Constant value 3
'''

AP_ORIENTATION = None
''' Constant value 4
'''

AP_PANNING = None
''' Constant value 1
'''

AP_PITCH = None
''' Constant value 2
'''

AP_VOLUME = None
''' Constant value 0
'''

CHANNELS_INVALID = None
''' Constant value 0
'''

CHANNELS_MONO = None
''' Constant value 1
'''

CHANNELS_STEREO = None
''' Constant value 2
'''

CHANNELS_STEREO_LFE = None
''' Constant value 3
'''

CHANNELS_SURROUND4 = None
''' Constant value 4
'''

CHANNELS_SURROUND5 = None
''' Constant value 5
'''

CHANNELS_SURROUND51 = None
''' Constant value 6
'''

CHANNELS_SURROUND61 = None
''' Constant value 7
'''

CHANNELS_SURROUND71 = None
''' Constant value 8
'''

CODEC_AAC = None
''' Constant value 1
'''

CODEC_AC3 = None
''' Constant value 2
'''

CODEC_FLAC = None
''' Constant value 3
'''

CODEC_INVALID = None
''' Constant value 0
'''

CODEC_MP2 = None
''' Constant value 4
'''

CODEC_MP3 = None
''' Constant value 5
'''

CODEC_OPUS = None
''' Constant value 8
'''

CODEC_PCM = None
''' Constant value 6
'''

CODEC_VORBIS = None
''' Constant value 7
'''

CONTAINER_AC3 = None
''' Constant value 1
'''

CONTAINER_FLAC = None
''' Constant value 2
'''

CONTAINER_INVALID = None
''' Constant value 0
'''

CONTAINER_MATROSKA = None
''' Constant value 3
'''

CONTAINER_MP2 = None
''' Constant value 4
'''

CONTAINER_MP3 = None
''' Constant value 5
'''

CONTAINER_OGG = None
''' Constant value 6
'''

CONTAINER_WAV = None
''' Constant value 7
'''

DISTANCE_MODEL_EXPONENT = None
''' Constant value 5
'''

DISTANCE_MODEL_EXPONENT_CLAMPED = None
''' Constant value 6
'''

DISTANCE_MODEL_INVALID = None
''' Constant value 0
'''

DISTANCE_MODEL_INVERSE = None
''' Constant value 1
'''

DISTANCE_MODEL_INVERSE_CLAMPED = None
''' Constant value 2
'''

DISTANCE_MODEL_LINEAR = None
''' Constant value 3
'''

DISTANCE_MODEL_LINEAR_CLAMPED = None
''' Constant value 4
'''

FORMAT_FLOAT32 = None
''' Constant value 36
'''

FORMAT_FLOAT64 = None
''' Constant value 40
'''

FORMAT_INVALID = None
''' Constant value 0
'''

FORMAT_S16 = None
''' Constant value 18
'''

FORMAT_S24 = None
''' Constant value 19
'''

FORMAT_S32 = None
''' Constant value 20
'''

FORMAT_U8 = None
''' Constant value 1
'''

RATE_11025 = None
''' Constant value 11025
'''

RATE_16000 = None
''' Constant value 16000
'''

RATE_192000 = None
''' Constant value 192000
'''

RATE_22050 = None
''' Constant value 22050
'''

RATE_32000 = None
''' Constant value 32000
'''

RATE_44100 = None
''' Constant value 44100
'''

RATE_48000 = None
''' Constant value 48000
'''

RATE_8000 = None
''' Constant value 8000
'''

RATE_88200 = None
''' Constant value 88200
'''

RATE_96000 = None
''' Constant value 96000
'''

RATE_INVALID = None
''' Constant value 0
'''

STATUS_INVALID = None
''' Constant value 0
'''

STATUS_PAUSED = None
''' Constant value 2
'''

STATUS_PLAYING = None
''' Constant value 1
'''

STATUS_STOPPED = None
''' Constant value 3
'''



import sys
import typing
import bpy.types

GenericType = typing.TypeVar("GenericType")


def fbx(override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: typing.Optional[bool] = None,
        *,
        filepath: typing.Union[str, typing.Any] = "",
        check_existing: typing.Union[bool, typing.Any] = True,
        filter_glob: typing.Union[str, typing.Any] = "*.fbx",
        use_selection: typing.Union[bool, typing.Any] = False,
        use_visible: typing.Union[bool, typing.Any] = False,
        use_active_collection: typing.Union[bool, typing.Any] = False,
        global_scale: typing.Optional[typing.Any] = 1.0,
        apply_unit_scale: typing.Union[bool, typing.Any] = True,
        apply_scale_options: typing.Optional[typing.Any] = 'FBX_SCALE_NONE',
        use_space_transform: typing.Union[bool, typing.Any] = True,
        bake_space_transform: typing.Union[bool, typing.Any] = False,
        object_types: typing.Optional[typing.Any] = {
            'ARMATURE', 'CAMERA', 'EMPTY', 'LIGHT', 'MESH', 'OTHER'
        },
        use_mesh_modifiers: typing.Union[bool, typing.Any] = True,
        use_mesh_modifiers_render: typing.Union[bool, typing.Any] = True,
        mesh_smooth_type: typing.Optional[typing.Any] = 'OFF',
        colors_type: typing.Optional[typing.Any] = 'SRGB',
        prioritize_active_color: typing.Union[bool, typing.Any] = False,
        use_subsurf: typing.Union[bool, typing.Any] = False,
        use_mesh_edges: typing.Union[bool, typing.Any] = False,
        use_tspace: typing.Union[bool, typing.Any] = False,
        use_triangles: typing.Union[bool, typing.Any] = False,
        use_custom_props: typing.Union[bool, typing.Any] = False,
        add_leaf_bones: typing.Union[bool, typing.Any] = True,
        primary_bone_axis: typing.Optional[typing.Any] = 'Y',
        secondary_bone_axis: typing.Optional[typing.Any] = 'X',
        use_armature_deform_only: typing.Union[bool, typing.Any] = False,
        armature_nodetype: typing.Optional[typing.Any] = 'NULL',
        bake_anim: typing.Union[bool, typing.Any] = True,
        bake_anim_use_all_bones: typing.Union[bool, typing.Any] = True,
        bake_anim_use_nla_strips: typing.Union[bool, typing.Any] = True,
        bake_anim_use_all_actions: typing.Union[bool, typing.Any] = True,
        bake_anim_force_startend_keying: typing.Union[bool, typing.Any] = True,
        bake_anim_step: typing.Optional[typing.Any] = 1.0,
        bake_anim_simplify_factor: typing.Optional[typing.Any] = 1.0,
        path_mode: typing.Optional[typing.Any] = 'AUTO',
        embed_textures: typing.Union[bool, typing.Any] = False,
        batch_mode: typing.Optional[typing.Any] = 'OFF',
        use_batch_own_dir: typing.Union[bool, typing.Any] = True,
        use_metadata: typing.Union[bool, typing.Any] = True,
        axis_forward: typing.Optional[typing.Any] = '-Z',
        axis_up: typing.Optional[typing.Any] = 'Y'):
    ''' Write a FBX file

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: typing.Optional[bool]
    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: typing.Union[str, typing.Any]
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: typing.Union[bool, typing.Any]
    :param filter_glob: filter_glob
    :type filter_glob: typing.Union[str, typing.Any]
    :param use_selection: Selected Objects, Export selected and visible objects only
    :type use_selection: typing.Union[bool, typing.Any]
    :param use_visible: Visible Objects, Export visible objects only
    :type use_visible: typing.Union[bool, typing.Any]
    :param use_active_collection: Active Collection, Export only objects from the active collection (and its children)
    :type use_active_collection: typing.Union[bool, typing.Any]
    :param global_scale: Scale, Scale all data (Some importers do not support scaled armatures!)
    :type global_scale: typing.Optional[typing.Any]
    :param apply_unit_scale: Apply Unit, Take into account current Blender units settings (if unset, raw Blender Units values are used as-is)
    :type apply_unit_scale: typing.Union[bool, typing.Any]
    :param apply_scale_options: Apply Scalings, How to apply custom and units scalings in generated FBX file (Blender uses FBX scale to detect units on import, but many other applications do not handle the same way) * ``FBX_SCALE_NONE`` All Local -- Apply custom scaling and units scaling to each object transformation, FBX scale remains at 1.0. * ``FBX_SCALE_UNITS`` FBX Units Scale -- Apply custom scaling to each object transformation, and units scaling to FBX scale. * ``FBX_SCALE_CUSTOM`` FBX Custom Scale -- Apply custom scaling to FBX scale, and units scaling to each object transformation. * ``FBX_SCALE_ALL`` FBX All -- Apply custom scaling and units scaling to FBX scale.
    :type apply_scale_options: typing.Optional[typing.Any]
    :param use_space_transform: Use Space Transform, Apply global space transform to the object rotations. When disabled only the axis space is written to the file and all object transforms are left as-is
    :type use_space_transform: typing.Union[bool, typing.Any]
    :param bake_space_transform: Apply Transform, Bake space transform into object data, avoids getting unwanted rotations to objects when target space is not aligned with Blender's space (WARNING! experimental option, use at own risk, known to be broken with armatures/animations)
    :type bake_space_transform: typing.Union[bool, typing.Any]
    :param object_types: Object Types, Which kind of object to export * ``EMPTY`` Empty. * ``CAMERA`` Camera. * ``LIGHT`` Lamp. * ``ARMATURE`` Armature -- WARNING: not supported in dupli/group instances. * ``MESH`` Mesh. * ``OTHER`` Other -- Other geometry types, like curve, metaball, etc. (converted to meshes).
    :type object_types: typing.Optional[typing.Any]
    :param use_mesh_modifiers: prevents exporting shape keys
    :type use_mesh_modifiers: typing.Union[bool, typing.Any]
    :param use_mesh_modifiers_render: Use Modifiers Render Setting, Use render settings when applying modifiers to mesh objects (DISABLED in Blender 2.8)
    :type use_mesh_modifiers_render: typing.Union[bool, typing.Any]
    :param mesh_smooth_type: Smoothing, Export smoothing information (prefer 'Normals Only' option if your target importer understand split normals) * ``OFF`` Normals Only -- Export only normals instead of writing edge or face smoothing data. * ``FACE`` Face -- Write face smoothing. * ``EDGE`` Edge -- Write edge smoothing.
    :type mesh_smooth_type: typing.Optional[typing.Any]
    :param colors_type: Vertex Colors, Export vertex color attributes * ``NONE`` None -- Do not export color attributes. * ``SRGB`` sRGB -- Export colors in sRGB color space. * ``LINEAR`` Linear -- Export colors in linear color space.
    :type colors_type: typing.Optional[typing.Any]
    :param prioritize_active_color: Prioritize Active Color, Make sure active color will be exported first. Could be important since some other software can discard other color attributes besides the first one
    :type prioritize_active_color: typing.Union[bool, typing.Any]
    :param use_subsurf: Export Subdivision Surface, Export the last Catmull-Rom subdivision modifier as FBX subdivision (does not apply the modifier even if 'Apply Modifiers' is enabled)
    :type use_subsurf: typing.Union[bool, typing.Any]
    :param use_mesh_edges: Loose Edges, Export loose edges (as two-vertices polygons)
    :type use_mesh_edges: typing.Union[bool, typing.Any]
    :param use_tspace: Tangent Space, Add binormal and tangent vectors, together with normal they form the tangent space (will only work correctly with tris/quads only meshes!)
    :type use_tspace: typing.Union[bool, typing.Any]
    :param use_triangles: Triangulate Faces, Convert all faces to triangles
    :type use_triangles: typing.Union[bool, typing.Any]
    :param use_custom_props: Custom Properties, Export custom properties
    :type use_custom_props: typing.Union[bool, typing.Any]
    :param add_leaf_bones: Add Leaf Bones, Append a final bone to the end of each chain to specify last bone length (use this when you intend to edit the armature from exported data)
    :type add_leaf_bones: typing.Union[bool, typing.Any]
    :param primary_bone_axis: Primary Bone Axis
    :type primary_bone_axis: typing.Optional[typing.Any]
    :param secondary_bone_axis: Secondary Bone Axis
    :type secondary_bone_axis: typing.Optional[typing.Any]
    :param use_armature_deform_only: Only Deform Bones, Only write deforming bones (and non-deforming ones when they have deforming children)
    :type use_armature_deform_only: typing.Union[bool, typing.Any]
    :param armature_nodetype: Armature FBXNode Type, FBX type of node (object) used to represent Blender's armatures (use the Null type unless you experience issues with the other app, as other choices may not import back perfectly into Blender...) * ``NULL`` Null -- 'Null' FBX node, similar to Blender's Empty (default). * ``ROOT`` Root -- 'Root' FBX node, supposed to be the root of chains of bones.... * ``LIMBNODE`` LimbNode -- 'LimbNode' FBX node, a regular joint between two bones....
    :type armature_nodetype: typing.Optional[typing.Any]
    :param bake_anim: Baked Animation, Export baked keyframe animation
    :type bake_anim: typing.Union[bool, typing.Any]
    :param bake_anim_use_all_bones: Key All Bones, Force exporting at least one key of animation for all bones (needed with some target applications, like UE4)
    :type bake_anim_use_all_bones: typing.Union[bool, typing.Any]
    :param bake_anim_use_nla_strips: NLA Strips, Export each non-muted NLA strip as a separated FBX's AnimStack, if any, instead of global scene animation
    :type bake_anim_use_nla_strips: typing.Union[bool, typing.Any]
    :param bake_anim_use_all_actions: All Actions, Export each action as a separated FBX's AnimStack, instead of global scene animation (note that animated objects will get all actions compatible with them, others will get no animation at all)
    :type bake_anim_use_all_actions: typing.Union[bool, typing.Any]
    :param bake_anim_force_startend_keying: Force Start/End Keying, Always add a keyframe at start and end of actions for animated channels
    :type bake_anim_force_startend_keying: typing.Union[bool, typing.Any]
    :param bake_anim_step: Sampling Rate, How often to evaluate animated values (in frames)
    :type bake_anim_step: typing.Optional[typing.Any]
    :param bake_anim_simplify_factor: Simplify, How much to simplify baked values (0.0 to disable, the higher the more simplified)
    :type bake_anim_simplify_factor: typing.Optional[typing.Any]
    :param path_mode: Path Mode, Method used to reference paths * ``AUTO`` Auto -- Use relative paths with subdirectories only. * ``ABSOLUTE`` Absolute -- Always write absolute paths. * ``RELATIVE`` Relative -- Always write relative paths (where possible). * ``MATCH`` Match -- Match absolute/relative setting with input path. * ``STRIP`` Strip Path -- Filename only. * ``COPY`` Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Optional[typing.Any]
    :param embed_textures: Embed Textures, Embed textures in FBX binary file (only for "Copy" path mode!)
    :type embed_textures: typing.Union[bool, typing.Any]
    :param batch_mode: Batch Mode * ``OFF`` Off -- Active scene to file. * ``SCENE`` Scene -- Each scene as a file. * ``COLLECTION`` Collection -- Each collection (data-block ones) as a file, does not include content of children collections. * ``SCENE_COLLECTION`` Scene Collections -- Each collection (including master, non-data-block ones) of each scene as a file, including content from children collections. * ``ACTIVE_SCENE_COLLECTION`` Active Scene Collections -- Each collection (including master, non-data-block one) of the active scene as a file, including content from children collections.
    :type batch_mode: typing.Optional[typing.Any]
    :param use_batch_own_dir: Batch Own Dir, Create a dir for each exported file
    :type use_batch_own_dir: typing.Union[bool, typing.Any]
    :param use_metadata: Use Metadata
    :type use_metadata: typing.Union[bool, typing.Any]
    :param axis_forward: Forward
    :type axis_forward: typing.Optional[typing.Any]
    :param axis_up: Up
    :type axis_up: typing.Optional[typing.Any]
    '''

    pass


def gltf(
        override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: typing.Optional[bool] = None,
        *,
        filepath: typing.Union[str, typing.Any] = "",
        check_existing: typing.Union[bool, typing.Any] = True,
        convert_lighting_mode: typing.Optional[typing.Any] = 'SPEC',
        gltf_export_id: typing.Union[str, typing.Any] = "",
        export_format: typing.Optional[typing.Any] = 'GLB',
        ui_tab: typing.Optional[typing.Any] = 'GENERAL',
        export_copyright: typing.Union[str, typing.Any] = "",
        export_image_format: typing.Optional[typing.Any] = 'AUTO',
        export_texture_dir: typing.Union[str, typing.Any] = "",
        export_jpeg_quality: typing.Optional[typing.Any] = 75,
        export_keep_originals: typing.Union[bool, typing.Any] = False,
        export_texcoords: typing.Union[bool, typing.Any] = True,
        export_normals: typing.Union[bool, typing.Any] = True,
        export_draco_mesh_compression_enable: typing.Union[bool, typing.
                                                           Any] = False,
        export_draco_mesh_compression_level: typing.Optional[typing.Any] = 6,
        export_draco_position_quantization: typing.Optional[typing.Any] = 14,
        export_draco_normal_quantization: typing.Optional[typing.Any] = 10,
        export_draco_texcoord_quantization: typing.Optional[typing.Any] = 12,
        export_draco_color_quantization: typing.Optional[typing.Any] = 10,
        export_draco_generic_quantization: typing.Optional[typing.Any] = 12,
        export_tangents: typing.Union[bool, typing.Any] = False,
        export_materials: typing.Optional[typing.Any] = 'EXPORT',
        export_original_specular: typing.Union[bool, typing.Any] = False,
        export_colors: typing.Union[bool, typing.Any] = True,
        export_attributes: typing.Union[bool, typing.Any] = False,
        use_mesh_edges: typing.Union[bool, typing.Any] = False,
        use_mesh_vertices: typing.Union[bool, typing.Any] = False,
        export_cameras: typing.Union[bool, typing.Any] = False,
        use_selection: typing.Union[bool, typing.Any] = False,
        use_visible: typing.Union[bool, typing.Any] = False,
        use_renderable: typing.Union[bool, typing.Any] = False,
        use_active_collection_with_nested: typing.Union[bool, typing.
                                                        Any] = True,
        use_active_collection: typing.Union[bool, typing.Any] = False,
        use_active_scene: typing.Union[bool, typing.Any] = False,
        export_extras: typing.Union[bool, typing.Any] = False,
        export_yup: typing.Union[bool, typing.Any] = True,
        export_apply: typing.Union[bool, typing.Any] = False,
        export_animations: typing.Union[bool, typing.Any] = True,
        export_frame_range: typing.Union[bool, typing.Any] = False,
        export_frame_step: typing.Optional[typing.Any] = 1,
        export_force_sampling: typing.Union[bool, typing.Any] = True,
        export_animation_mode: typing.Optional[typing.Any] = 'ACTIONS',
        export_nla_strips_merged_animation_name: typing.
        Union[str, typing.Any] = "Animation",
        export_def_bones: typing.Union[bool, typing.Any] = False,
        export_hierarchy_flatten_bones: typing.Union[bool, typing.Any] = False,
        export_optimize_animation_size: typing.Union[bool, typing.Any] = True,
        export_optimize_animation_keep_anim_armature: typing.
        Union[bool, typing.Any] = True,
        export_optimize_animation_keep_anim_object: typing.Union[bool, typing.
                                                                 Any] = False,
        export_negative_frame: typing.Optional[typing.Any] = 'SLIDE',
        export_anim_slide_to_zero: typing.Union[bool, typing.Any] = False,
        export_bake_animation: typing.Union[bool, typing.Any] = False,
        export_anim_single_armature: typing.Union[bool, typing.Any] = True,
        export_reset_pose_bones: typing.Union[bool, typing.Any] = True,
        export_current_frame: typing.Union[bool, typing.Any] = False,
        export_rest_position_armature: typing.Union[bool, typing.Any] = True,
        export_anim_scene_split_object: typing.Union[bool, typing.Any] = True,
        export_skins: typing.Union[bool, typing.Any] = True,
        export_all_influences: typing.Union[bool, typing.Any] = False,
        export_morph: typing.Union[bool, typing.Any] = True,
        export_morph_normal: typing.Union[bool, typing.Any] = True,
        export_morph_tangent: typing.Union[bool, typing.Any] = False,
        export_morph_animation: typing.Union[bool, typing.Any] = True,
        export_lights: typing.Union[bool, typing.Any] = False,
        will_save_settings: typing.Union[bool, typing.Any] = False,
        filter_glob: typing.Union[str, typing.Any] = "*.glb"):
    ''' Export scene as glTF 2.0 file

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: typing.Optional[bool]
    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: typing.Union[str, typing.Any]
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: typing.Union[bool, typing.Any]
    :param convert_lighting_mode: Lighting Mode, Optional backwards compatibility for non-standard render engines. Applies to lights * ``SPEC`` Standard -- Physically-based glTF lighting units (cd, lx, nt). * ``COMPAT`` Unitless -- Non-physical, unitless lighting. Useful when exposure controls are not available. * ``RAW`` Raw (Deprecated) -- Blender lighting strengths with no conversion.
    :type convert_lighting_mode: typing.Optional[typing.Any]
    :param gltf_export_id: Identifier, Identifier of caller (in case of add-on calling this exporter). Can be useful in case of Extension added by other add-ons
    :type gltf_export_id: typing.Union[str, typing.Any]
    :param export_format: Format, Output format and embedding options. Binary is most efficient, but JSON (embedded or separate) may be easier to edit later * ``GLB`` glTF Binary (.glb) -- Exports a single file, with all data packed in binary form. Most efficient and portable, but more difficult to edit later. * ``GLTF_SEPARATE`` glTF Separate (.gltf + .bin + textures) -- Exports multiple files, with separate JSON, binary and texture data. Easiest to edit later. * ``GLTF_EMBEDDED`` glTF Embedded (.gltf) -- Exports a single file, with all data packed in JSON. Less efficient than binary, but easier to edit later.
    :type export_format: typing.Optional[typing.Any]
    :param ui_tab: ui_tab, Export setting categories * ``GENERAL`` General -- General settings. * ``MESHES`` Meshes -- Mesh settings. * ``OBJECTS`` Objects -- Object settings. * ``ANIMATION`` Animation -- Animation settings.
    :type ui_tab: typing.Optional[typing.Any]
    :param export_copyright: Copyright, Legal rights and conditions for the model
    :type export_copyright: typing.Union[str, typing.Any]
    :param export_image_format: Images, Output format for images. PNG is lossless and generally preferred, but JPEG might be preferable for web applications due to the smaller file size. Alternatively they can be omitted if they are not needed * ``AUTO`` Automatic -- Save PNGs as PNGs and JPEGs as JPEGs. If neither one, use PNG. * ``JPEG`` JPEG Format (.jpg) -- Save images as JPEGs. (Images that need alpha are saved as PNGs though.) Be aware of a possible loss in quality. * ``NONE`` None -- Don't export images.
    :type export_image_format: typing.Optional[typing.Any]
    :param export_texture_dir: Textures, Folder to place texture files in. Relative to the .gltf file
    :type export_texture_dir: typing.Union[str, typing.Any]
    :param export_jpeg_quality: JPEG quality, Quality of JPEG export
    :type export_jpeg_quality: typing.Optional[typing.Any]
    :param export_keep_originals: if you use more than one texture, where pbr standard requires only one, only one texture will be used. This can lead to unexpected results
    :type export_keep_originals: typing.Union[bool, typing.Any]
    :param export_texcoords: UVs, Export UVs (texture coordinates) with meshes
    :type export_texcoords: typing.Union[bool, typing.Any]
    :param export_normals: Normals, Export vertex normals with meshes
    :type export_normals: typing.Union[bool, typing.Any]
    :param export_draco_mesh_compression_enable: Draco mesh compression, Compress mesh using Draco
    :type export_draco_mesh_compression_enable: typing.Union[bool, typing.Any]
    :param export_draco_mesh_compression_level: Compression level, Compression level (0 = most speed, 6 = most compression, higher values currently not supported)
    :type export_draco_mesh_compression_level: typing.Optional[typing.Any]
    :param export_draco_position_quantization: Position quantization bits, Quantization bits for position values (0 = no quantization)
    :type export_draco_position_quantization: typing.Optional[typing.Any]
    :param export_draco_normal_quantization: Normal quantization bits, Quantization bits for normal values (0 = no quantization)
    :type export_draco_normal_quantization: typing.Optional[typing.Any]
    :param export_draco_texcoord_quantization: Texcoord quantization bits, Quantization bits for texture coordinate values (0 = no quantization)
    :type export_draco_texcoord_quantization: typing.Optional[typing.Any]
    :param export_draco_color_quantization: Color quantization bits, Quantization bits for color values (0 = no quantization)
    :type export_draco_color_quantization: typing.Optional[typing.Any]
    :param export_draco_generic_quantization: Generic quantization bits, Quantization bits for generic values like weights or joints (0 = no quantization)
    :type export_draco_generic_quantization: typing.Optional[typing.Any]
    :param export_tangents: Tangents, Export vertex tangents with meshes
    :type export_tangents: typing.Union[bool, typing.Any]
    :param export_materials: Materials, Export materials * ``EXPORT`` Export -- Export all materials used by included objects. * ``PLACEHOLDER`` Placeholder -- Do not export materials, but write multiple primitive groups per mesh, keeping material slot information. * ``NONE`` No export -- Do not export materials, and combine mesh primitive groups, losing material slot information.
    :type export_materials: typing.Optional[typing.Any]
    :param export_original_specular: Export original PBR Specular, Export original glTF PBR Specular, instead of Blender Principled Shader Specular
    :type export_original_specular: typing.Union[bool, typing.Any]
    :param export_colors: Vertex Colors, Export vertex colors with meshes
    :type export_colors: typing.Union[bool, typing.Any]
    :param export_attributes: Attributes, Export Attributes (when starting with underscore)
    :type export_attributes: typing.Union[bool, typing.Any]
    :param use_mesh_edges: Loose Edges, Export loose edges as lines, using the material from the first material slot
    :type use_mesh_edges: typing.Union[bool, typing.Any]
    :param use_mesh_vertices: Loose Points, Export loose points as glTF points, using the material from the first material slot
    :type use_mesh_vertices: typing.Union[bool, typing.Any]
    :param export_cameras: Cameras, Export cameras
    :type export_cameras: typing.Union[bool, typing.Any]
    :param use_selection: Selected Objects, Export selected objects only
    :type use_selection: typing.Union[bool, typing.Any]
    :param use_visible: Visible Objects, Export visible objects only
    :type use_visible: typing.Union[bool, typing.Any]
    :param use_renderable: Renderable Objects, Export renderable objects only
    :type use_renderable: typing.Union[bool, typing.Any]
    :param use_active_collection_with_nested: Include Nested Collections, Include active collection and nested collections
    :type use_active_collection_with_nested: typing.Union[bool, typing.Any]
    :param use_active_collection: Active Collection, Export objects in the active collection only
    :type use_active_collection: typing.Union[bool, typing.Any]
    :param use_active_scene: Active Scene, Export active scene only
    :type use_active_scene: typing.Union[bool, typing.Any]
    :param export_extras: Custom Properties, Export custom properties as glTF extras
    :type export_extras: typing.Union[bool, typing.Any]
    :param export_yup: +Y Up, Export using glTF convention, +Y up
    :type export_yup: typing.Union[bool, typing.Any]
    :param export_apply: prevents exporting shape keys
    :type export_apply: typing.Union[bool, typing.Any]
    :param export_animations: Animations, Exports active actions and NLA tracks as glTF animations
    :type export_animations: typing.Union[bool, typing.Any]
    :param export_frame_range: Limit to Playback Range, Clips animations to selected playback range
    :type export_frame_range: typing.Union[bool, typing.Any]
    :param export_frame_step: Sampling Rate, How often to evaluate animated values (in frames)
    :type export_frame_step: typing.Optional[typing.Any]
    :param export_force_sampling: Always Sample Animations, Apply sampling to all animations
    :type export_force_sampling: typing.Union[bool, typing.Any]
    :param export_animation_mode: Animation mode, Export Animation mode * ``ACTIONS`` Actions -- Export actions (actives and on NLA tracks) as separate animations. * ``ACTIVE_ACTIONS`` Active actions merged -- All the currently assigned actions become one glTF animation. * ``NLA_TRACKS`` NLA Tracks -- Export individual NLA Tracks as separate animation. * ``SCENE`` Scene -- Export baked scene as a single animation.
    :type export_animation_mode: typing.Optional[typing.Any]
    :param export_nla_strips_merged_animation_name: Merged Animation Name, Name of single glTF animation to be exported
    :type export_nla_strips_merged_animation_name: typing.Union[str, typing.Any]
    :param export_def_bones: Export Deformation Bones Only, Export Deformation bones only
    :type export_def_bones: typing.Union[bool, typing.Any]
    :param export_hierarchy_flatten_bones: Flatten Bone Hierarchy, Flatten Bone Hierarchy. Usefull in case of non decomposable TRS matrix
    :type export_hierarchy_flatten_bones: typing.Union[bool, typing.Any]
    :param export_optimize_animation_size: Optimize Animation Size, Reduce exported file size by removing duplicate keyframes (can cause problems with stepped animation)
    :type export_optimize_animation_size: typing.Union[bool, typing.Any]
    :param export_optimize_animation_keep_anim_armature: Force keeping channel for armature / bones, if all keyframes are identical in a rig force keeping the minimal animation
    :type export_optimize_animation_keep_anim_armature: typing.Union[bool, typing.Any]
    :param export_optimize_animation_keep_anim_object: Force keeping channel for objects, if all keyframes are identical for object transformations force keeping the minimal animation
    :type export_optimize_animation_keep_anim_object: typing.Union[bool, typing.Any]
    :param export_negative_frame: Negative Frames, Negative Frames are slided or cropped * ``SLIDE`` Slide -- Slide animation to start at frame 0. * ``CROP`` Crop -- Keep only frames above frame 0.
    :type export_negative_frame: typing.Optional[typing.Any]
    :param export_anim_slide_to_zero: Set all glTF Animation starting at 0, Set all glTF animation starting at 0.0s. Can be usefull for looping animations
    :type export_anim_slide_to_zero: typing.Union[bool, typing.Any]
    :param export_bake_animation: Bake All Objects Animations, Force exporting animation on every objects. Can be usefull when using constraints or driver. Also useful when exporting only selection
    :type export_bake_animation: typing.Union[bool, typing.Any]
    :param export_anim_single_armature: Option does not support exports including multiple armatures
    :type export_anim_single_armature: typing.Union[bool, typing.Any]
    :param export_reset_pose_bones: Reset pose bones between actions, Reset pose bones between each action exported. This is needed when some bones are not keyed on some animations
    :type export_reset_pose_bones: typing.Union[bool, typing.Any]
    :param export_current_frame: Use Current Frame as Object Rest Transformations, Export the scene in the current animation frame. When off, frame O is used as rest transformations for objects
    :type export_current_frame: typing.Union[bool, typing.Any]
    :param export_rest_position_armature: Use Rest Position Armature, Export armatures using rest position as joins rest pose. When off, current frame pose is used as rest pose
    :type export_rest_position_armature: typing.Union[bool, typing.Any]
    :param export_anim_scene_split_object: Split Animation by Object, Export Scene as seen in Viewport, But split animation by Object
    :type export_anim_scene_split_object: typing.Union[bool, typing.Any]
    :param export_skins: Skinning, Export skinning (armature) data
    :type export_skins: typing.Union[bool, typing.Any]
    :param export_all_influences: Include All Bone Influences, Allow >4 joint vertex influences. Models may appear incorrectly in many viewers
    :type export_all_influences: typing.Union[bool, typing.Any]
    :param export_morph: Shape Keys, Export shape keys (morph targets)
    :type export_morph: typing.Union[bool, typing.Any]
    :param export_morph_normal: Shape Key Normals, Export vertex normals with shape keys (morph targets)
    :type export_morph_normal: typing.Union[bool, typing.Any]
    :param export_morph_tangent: Shape Key Tangents, Export vertex tangents with shape keys (morph targets)
    :type export_morph_tangent: typing.Union[bool, typing.Any]
    :param export_morph_animation: Shape Key Animations, Export shape keys animations (morph targets)
    :type export_morph_animation: typing.Union[bool, typing.Any]
    :param export_lights: Punctual Lights, Export directional, point, and spot lights. Uses "KHR_lights_punctual" glTF extension
    :type export_lights: typing.Union[bool, typing.Any]
    :param will_save_settings: Remember Export Settings, Store glTF export settings in the Blender project
    :type will_save_settings: typing.Union[bool, typing.Any]
    :param filter_glob: filter_glob
    :type filter_glob: typing.Union[str, typing.Any]
    '''

    pass


def obj(override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: typing.Optional[bool] = None,
        *,
        filepath: typing.Union[str, typing.Any] = "",
        check_existing: typing.Union[bool, typing.Any] = True,
        filter_glob: typing.Union[str, typing.Any] = "*.obj;*.mtl",
        use_selection: typing.Union[bool, typing.Any] = False,
        use_animation: typing.Union[bool, typing.Any] = False,
        use_mesh_modifiers: typing.Union[bool, typing.Any] = True,
        use_edges: typing.Union[bool, typing.Any] = True,
        use_smooth_groups: typing.Union[bool, typing.Any] = False,
        use_smooth_groups_bitflags: typing.Union[bool, typing.Any] = False,
        use_normals: typing.Union[bool, typing.Any] = True,
        use_uvs: typing.Union[bool, typing.Any] = True,
        use_materials: typing.Union[bool, typing.Any] = True,
        use_triangles: typing.Union[bool, typing.Any] = False,
        use_nurbs: typing.Union[bool, typing.Any] = False,
        use_vertex_groups: typing.Union[bool, typing.Any] = False,
        use_blen_objects: typing.Union[bool, typing.Any] = True,
        group_by_object: typing.Union[bool, typing.Any] = False,
        group_by_material: typing.Union[bool, typing.Any] = False,
        keep_vertex_order: typing.Union[bool, typing.Any] = False,
        global_scale: typing.Optional[typing.Any] = 1.0,
        path_mode: typing.Optional[typing.Any] = 'AUTO',
        axis_forward: typing.Optional[typing.Any] = '-Z',
        axis_up: typing.Optional[typing.Any] = 'Y'):
    ''' Save a Wavefront OBJ File

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: typing.Optional[bool]
    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: typing.Union[str, typing.Any]
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: typing.Union[bool, typing.Any]
    :param filter_glob: filter_glob
    :type filter_glob: typing.Union[str, typing.Any]
    :param use_selection: Selection Only, Export selected objects only
    :type use_selection: typing.Union[bool, typing.Any]
    :param use_animation: Animation, Write out an OBJ for each frame
    :type use_animation: typing.Union[bool, typing.Any]
    :param use_mesh_modifiers: Apply Modifiers, Apply modifiers
    :type use_mesh_modifiers: typing.Union[bool, typing.Any]
    :param use_edges: Include Edges
    :type use_edges: typing.Union[bool, typing.Any]
    :param use_smooth_groups: Smooth Groups, Write sharp edges as smooth groups
    :type use_smooth_groups: typing.Union[bool, typing.Any]
    :param use_smooth_groups_bitflags: Bitflag Smooth Groups, Same as 'Smooth Groups', but generate smooth groups IDs as bitflags (produces at most 32 different smooth groups, usually much less)
    :type use_smooth_groups_bitflags: typing.Union[bool, typing.Any]
    :param use_normals: Write Normals, Export one normal per vertex and per face, to represent flat faces and sharp edges
    :type use_normals: typing.Union[bool, typing.Any]
    :param use_uvs: Include UVs, Write out the active UV coordinates
    :type use_uvs: typing.Union[bool, typing.Any]
    :param use_materials: Write Materials, Write out the MTL file
    :type use_materials: typing.Union[bool, typing.Any]
    :param use_triangles: Triangulate Faces, Convert all faces to triangles
    :type use_triangles: typing.Union[bool, typing.Any]
    :param use_nurbs: Write Nurbs, Write nurbs curves as OBJ nurbs rather than converting to geometry
    :type use_nurbs: typing.Union[bool, typing.Any]
    :param use_vertex_groups: Polygroups
    :type use_vertex_groups: typing.Union[bool, typing.Any]
    :param use_blen_objects: OBJ Objects, Export Blender objects as OBJ objects
    :type use_blen_objects: typing.Union[bool, typing.Any]
    :param group_by_object: OBJ Groups, Export Blender objects as OBJ groups
    :type group_by_object: typing.Union[bool, typing.Any]
    :param group_by_material: Material Groups, Generate an OBJ group for each part of a geometry using a different material
    :type group_by_material: typing.Union[bool, typing.Any]
    :param keep_vertex_order: Keep Vertex Order
    :type keep_vertex_order: typing.Union[bool, typing.Any]
    :param global_scale: Scale
    :type global_scale: typing.Optional[typing.Any]
    :param path_mode: Path Mode, Method used to reference paths * ``AUTO`` Auto -- Use relative paths with subdirectories only. * ``ABSOLUTE`` Absolute -- Always write absolute paths. * ``RELATIVE`` Relative -- Always write relative paths (where possible). * ``MATCH`` Match -- Match absolute/relative setting with input path. * ``STRIP`` Strip Path -- Filename only. * ``COPY`` Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Optional[typing.Any]
    :param axis_forward: Forward
    :type axis_forward: typing.Optional[typing.Any]
    :param axis_up: Up
    :type axis_up: typing.Optional[typing.Any]
    '''

    pass


def x3d(override_context: typing.Union[typing.
                                       Dict, 'bpy.types.Context'] = None,
        execution_context: typing.Union[str, int] = None,
        undo: typing.Optional[bool] = None,
        *,
        filepath: typing.Union[str, typing.Any] = "",
        check_existing: typing.Union[bool, typing.Any] = True,
        filter_glob: typing.Union[str, typing.Any] = "*.x3d",
        use_selection: typing.Union[bool, typing.Any] = False,
        use_mesh_modifiers: typing.Union[bool, typing.Any] = True,
        use_triangulate: typing.Union[bool, typing.Any] = False,
        use_normals: typing.Union[bool, typing.Any] = False,
        use_compress: typing.Union[bool, typing.Any] = False,
        use_hierarchy: typing.Union[bool, typing.Any] = True,
        name_decorations: typing.Union[bool, typing.Any] = True,
        use_h3d: typing.Union[bool, typing.Any] = False,
        global_scale: typing.Optional[typing.Any] = 1.0,
        path_mode: typing.Optional[typing.Any] = 'AUTO',
        axis_forward: typing.Optional[typing.Any] = 'Z',
        axis_up: typing.Optional[typing.Any] = 'Y'):
    ''' Export selection to Extensible 3D file (.x3d)

    :type override_context: typing.Union[typing.Dict, 'bpy.types.Context']
    :type execution_context: typing.Union[str, int]
    :type undo: typing.Optional[bool]
    :param filepath: File Path, Filepath used for exporting the file
    :type filepath: typing.Union[str, typing.Any]
    :param check_existing: Check Existing, Check and warn on overwriting existing files
    :type check_existing: typing.Union[bool, typing.Any]
    :param filter_glob: filter_glob
    :type filter_glob: typing.Union[str, typing.Any]
    :param use_selection: Selection Only, Export selected objects only
    :type use_selection: typing.Union[bool, typing.Any]
    :param use_mesh_modifiers: Apply Modifiers, Use transformed mesh data from each object
    :type use_mesh_modifiers: typing.Union[bool, typing.Any]
    :param use_triangulate: Triangulate, Write quads into 'IndexedTriangleSet'
    :type use_triangulate: typing.Union[bool, typing.Any]
    :param use_normals: Normals, Write normals with geometry
    :type use_normals: typing.Union[bool, typing.Any]
    :param use_compress: Compress, Compress the exported file
    :type use_compress: typing.Union[bool, typing.Any]
    :param use_hierarchy: Hierarchy, Export parent child relationships
    :type use_hierarchy: typing.Union[bool, typing.Any]
    :param name_decorations: Name decorations, Add prefixes to the names of exported nodes to indicate their type
    :type name_decorations: typing.Union[bool, typing.Any]
    :param use_h3d: H3D Extensions, Export shaders for H3D
    :type use_h3d: typing.Union[bool, typing.Any]
    :param global_scale: Scale
    :type global_scale: typing.Optional[typing.Any]
    :param path_mode: Path Mode, Method used to reference paths * ``AUTO`` Auto -- Use relative paths with subdirectories only. * ``ABSOLUTE`` Absolute -- Always write absolute paths. * ``RELATIVE`` Relative -- Always write relative paths (where possible). * ``MATCH`` Match -- Match absolute/relative setting with input path. * ``STRIP`` Strip Path -- Filename only. * ``COPY`` Copy -- Copy the file to the destination path (or subdirectory).
    :type path_mode: typing.Optional[typing.Any]
    :param axis_forward: Forward
    :type axis_forward: typing.Optional[typing.Any]
    :param axis_up: Up
    :type axis_up: typing.Optional[typing.Any]
    '''

    pass



import os
import glob
import datetime
from setuptools import setup, find_packages

# module name
cur_dir = os.getcwd().replace("\\", "/")
blender_version = cur_dir.split('/')[-1].split('-')[-1]
module_name = f"fake-bpy-module-{blender_version}"

# release version
if "RELEASE_VERSION" in os.environ:
    print("Environment variable 'RELEASE_VERSION' exists, "
          "so use it as release version")
    release_version = os.environ["RELEASE_VERSION"]
else:
    print("Environment variable 'RELEASE_VERSION' does not exist, "
          "so use date as release version")
    release_version = datetime.datetime.today().strftime("%Y%m%d")

# long_description
try:
    readme_path = "{}/README.rst".format(
        os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"))
    with open(readme_path, "r", encoding="utf-8") as f:
        long_description = f.read()
except IOError:
    long_description = ""   # pylint: disable=C0103

# find python module.
py_modules = list(set(glob.glob("*.py")) - {"setup.py"})
py_modules = [os.path.splitext(m)[0] for m in py_modules]

# find python packages and setup package data
packages = find_packages()
package_data = {pkg: ["py.typed"] for pkg in packages}

setup(
    name=module_name,
    version=release_version,
    url="https://github.com/nutti/fake-bpy-module",
    author="nutti",
    author_email="nutti.metro@gmail.com",
    maintainer="nutti",
    maintainer_email="nutti.metro@gmail.com",
    description="Collection of the fake Blender Python API module for "
                "the code completion.",
    long_description=long_description,
    project_urls={
        "Bug Tracker": "https://github.com/nutti/fake-bpy-module/issues",
        "Documentation": "https://github.com/nutti/fake-bpy-module/blob/"
                         "master/README.md",
        "Source Code": "https://github.com/nutti/fake-bpy-module",
    },
    platforms=["Windows", "Linux", "Mac OS-X"],
    py_modules=py_modules,
    package_data=package_data,
    packages=packages,
    zip_safe=False,
    python_requires=">=3.7",
    license="MIT",
    classifiers=[
        "Topic :: Multimedia :: Graphics :: 3D Modeling",
        "Topic :: Multimedia :: Graphics :: 3D Rendering",
        "Topic :: Text Editors :: Integrated Development Environments (IDE)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ]
)
